RowLayerState

Struct RowLayerState 

struct RowLayerState {
    push_cmd_idx: usize,
    span: Option<Span>,
}
Each recorded layer by itself already stores a bounding box. However, we want to take this one step further and track a horizontal bounding box of each row band individually. This allows us to save a lot of work when clearing buffers during fine rasterization.

For example, let’s say that we are rendering a triangle with the points (0, 0), (500, 0) and (250, 250). The bounding box would be (0, 0) and (500, 500). However, the triangle gets much more narrow as we advance through the rows. If we just chose the coarse bounding box, we would always end up clearing a whole width of 500 pixels during fine rasterization. By tracking the bounding box per row, we can reduce the work to the absolute minimum necessary for each row.

Fields

push_cmd_idx: usize

The index in the command stream where the PushBuf command of the corresponding layer lives.

span: Option<Span>

The horizontal span of the layer.